/* 
 *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
 *    Copyright 2013, 2014 (c)
 * 
 *    file: print.c
 *    This file is part of XTV.
 *
 *    XTV is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    XTV is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with XTV.  If not, see <http://www.gnu.org/licenses/>.
 */    
#include "print.h"
#include "defs.h"

extern void exportTree();	//found in file.c
extern int getKey();

/************************************
 * This function will show the print
 * dialog box, including installed
 * printers. It takes control over
 * user input, then returns to main.
 * **********************************/
void showPrintDialogBox() {
  //find installed printers on system. If function returns error,
  //give user an error message and return to main.

//  if(findInstalledPrinters() != 0) {
  if(numPrinters <= 0) {
    msgBox("Can't find installed printers,\nOr system has no installed printers.", OK, ERROR);
    fprintf(log_file, "Can't find installed printers, or system has no installed printers.\n");
    refreshFileView();
    refreshDirView();
    return;
  }
  
  int x, y;
  int w = 40; int h = 14;
  int i;
  x = (SCREEN_H-h)/2;
  y = (SCREEN_W-w)/2;
  w += y; h += x;

  //DRAW WINDOW
  setScreenColors(WHITE, BGBLUE);
  drawBox(x, y, h, w, " PRINT DIRECTORY TREE ", YES);
  fprintf(stdout, "\e[%d;%dH", x+1, y+2);
  fprintf(stdout, "Select a printer:");
  fprintf(stdout, "\e[%d;%dH", h-1, y+2);
  fprintf(stdout, "[ENTER] = PRINT, [ANY KEY] = CANCEL");
  //PRINT FIRST OPTION AS HIGHLIGHTED
  setScreenColors(BLUE, BGWHITE);
  fprintf(stdout, "\e[%d;%dH", x+2, y+2);
  printf("%*s", (w-y-3), " ");
  fprintf(stdout, "\e[%d;%dH", x+2, y+2);
  printf("%s", printers->name);

  //OUTPUT REST OF PRINTERS LIST
  setScreenColors(WHITE, BGBLUE);
   for(i = 1; i < numPrinters; i++) {
    fprintf(stdout, "\e[%d;%dH", x+i+2, y+2);
    printf("%s", (printers+i)->name);
    //printf("%s", printers[i+1]->name);
  }
  fprintf(stdout, "\e[%d;%dH", x+2, w-y-2);

  int sel = 0;
  while(1) {
    char ch = getKey();
    switch(ch) {
      case(UP_KEY):
	//un-highlight current selection
	setScreenColors(WHITE, BGBLUE);
	fprintf(stdout, "\e[%d;%dH", x+sel+2, y+2);
	//printf("%s", printers[sel+1]->name);
	printf("%s", (printers+sel)->name);
	//printf("%*s", (w-y-strlen(printers[sel+1]->name)-1), " ");
	printf("%*s", (w-y-strlen((printers+sel)->name)-3), " ");
	//then update selection
	if(sel == 0) sel = numPrinters;
	sel--;
	//highlight new selection
	setScreenColors(BLUE, BGWHITE);
	fprintf(stdout, "\e[%d;%dH", x+sel+2, y+2);
	//printf("%s", printers[sel+1]->name);
	printf("%s", (printers+sel)->name);
	//printf("%*s", (w-y-strlen(printers[sel+1]->name)-1), " ");
	printf("%*s", (w-y-strlen((printers+sel)->name)-3), " ");
	break;
      case(DOWN_KEY):
	//un-highlight current selection
	setScreenColors(WHITE, BGBLUE);
	fprintf(stdout, "\e[%d;%dH", x+sel+2, y+2);
	//printf("%s", printers[sel+1]->name);
	printf("%s", (printers+sel)->name);
	//printf("%*s", (w-y-strlen(printers[sel+1]->name)-1), " ");
	printf("%*s", (w-y-strlen((printers+sel)->name)-3), " ");
	//then update selection
	if(sel == numPrinters - 1) sel = -1;
	sel++;
	//highlight new selection
	setScreenColors(BLUE, BGWHITE);
	fprintf(stdout, "\e[%d;%dH", x+sel+2, y+2);
	//printf("%s", printers[sel+1]->name);
	printf("%s", (printers+sel)->name);
	//printf("%*s", (w-y-strlen(printers[sel+1]->name)-1), " ");
	printf("%*s", (w-y-strlen((printers+sel)->name)-3), " ");
	break;
      case(ENTER_KEY):
	strcpy(input, "/tmp/xtv_print.tmp");
	exportTree(NO);
	selectedPrinter = sel;
	fprintf(log_file, "Printing directory tree to temp file '/tmp/xtv_print.tmp'..\n");
	printFile("/tmp/xtv_print.tmp", cwd);
      default:
	if(PRINTING) { PRINTING = 0; remove("/tmp/xtv_print.tmp"); }	//remove temp. print file
	freePrinters();
	setScreenColors(WHITE, BGBLUE);
	refreshFileView();
	refreshDirView();
	drawMenuBar(2, 2, SCREEN_W-2);							//draw main menu bar
	refreshBottomView();
	return;
    }//end switch
    
  }//end while
}


int checkPrintingState() {
 if(!PRINTING) return IPP_JOB_COMPLETED;
 cups_job_t *jobs;
 ipp_jstate_t state = IPP_JOB_COMPLETED;
 //int n = cupsGetJobs(&jobs, printers[selectedPrinter], 1, -1); //1 for current user, -1 for any state -- according to cups docs
 int n = cupsGetJobs(&jobs, (printers+selectedPrinter)->name, 1, -1); //1 for current user, -1 for any state -- according to cups docs
 int i;
 for(i = 0; i < n; i++)
  if(jobs[i].id == printingJob) {
   state = jobs[i].state; break;
  }
 
 cupsFreeJobs(n, jobs);
 return state;
}

/****************************************
 * This function sends the file passed to
 * it as *printFileName, with the passed
 * *printTitle as title, to the selected
 * printer no. saved in selectedPrinter.
 * **************************************/
int printFile(char *printFileName, char *printTitle) {
 if(!selectedPrinter) return 1; //error--> user didn't select printer!!

 //cups_dest_t *prn = cupsGetDest(printers[selectedPrinter]->name, NULL, numPrinters, printers);
 cups_dest_t *prn = printers+selectedPrinter;
 
 printingJob = cupsPrintFile(prn->name, printFileName, printTitle, prn->num_options, prn->options);
 PRINTING = 1;
 return 0;
}

void freePrinters() {
 cupsFreeDests(numPrinters, printers);
 //free(printerStatus);
}

/**********************************
 * This function queries the cups
 * library for printers installed
 * in the system. It returns 0 if
 * successful, 1 if it failed.
 * ********************************/
int findInstalledPrinters() {
 int i;
 cups_dest_t *prn;
 numPrinters = cupsGetDests(&printers); //query for installed printers
 if(numPrinters == 0) return 1;	//return error

 if(numPrinters > MAX_NUM_PRINTERS) numPrinters = MAX_NUM_PRINTERS;
 
 //printerStatus = (char *) malloc(MAX_NUM_PRINTERS+1);
 memset(printerStatus, ' ', MAX_NUM_PRINTERS);
 
 prn = printers;
 
 for(i = 0; i < numPrinters; i++, prn++) {
  //printerStatus[i] = cupsGetOption("printer-state", prn->num_options, prn->options);
  //if(!printerStatus[i]) printerStatus[i] = 'E';
  
  const char *c = cupsGetOption("printer-state", prn->num_options, prn->options);
  /*if(c == IPP_JOB_PENDING) printerStatus[i] = 'B';
  else if(c == IPP_JOB_HELD) printerStatus[i] = 'B';
  else if(c == IPP_JOB_PROCESSING) printerStatus[i] = 'B';
  else if(c == IPP_JOB_STOPPED) printerStatus[i] = 'S';
  else if(c == IPP_JOB_CANCELED) printerStatus[i] = 'I';
  else if(c == IPP_JOB_ABORTED) printerStatus[i] = 'I';
  else if(c == IPP_JOB_COMPLETED) printerStatus[i] = 'I';
  else printerStatus[i] = 'E';*/
  printerStatus[i] = c[0];
  /*if(prn->instance)
   printf("%s/%s\n", prn->name, prn->instance);
  else
   puts(prn->name);*/
 }//end for
 return 0; //return success
}//end function
